using System;
using System.ComponentModel;
using System.Collections.Generic;

namespace FeatureVoting {
	public class FeatureSummaryPropertyDescriptor : PropertyDescriptor {
		public const string STR_PropertyName = "% of average";

		public FeatureSummaryPropertyDescriptor(BindingList<Feature> featureList, BindingList<Vote> voteList)
			: base(STR_PropertyName, null) {
			this.featureList = featureList;
			this.voteList = voteList;
			voteList.ListChanged += new ListChangedEventHandler(voteList_ListChanged);
		}

		private BindingList<Feature> featureList;
		public BindingList<Feature> FeatureList {
			get {
				return featureList;
			}
		}

		private BindingList<Vote> voteList;
		public BindingList<Vote> VoteList {
			get {
				return voteList;
			}
		}

		void voteList_ListChanged(object sender, ListChangedEventArgs e) {
			if (e.ListChangedType == ListChangedType.ItemChanged) {
				PushChangeToAllFeatures( );
			}
		}

		private void PushChangeToAllFeatures( ) {
			foreach (Feature feature in featureList)
				feature.RaisePropertyChanged(STR_PropertyName);
		}

		public override void SetValue(object component, object value) { }

		public override object GetValue(object component) {
			Feature feature = (Feature) component;
			return CalcPercentOfAverage(feature);
		}

		private double CalcPercentOfAverage(Feature feature) {
			Dictionary<Feature, FeatureInfo> averageInfos = CalcYearAveragePerFeature( );
			if (averageInfos.ContainsKey(feature)) {
				double thisFeatureAverage = averageInfos[feature].Average;
				double overAllAverage = 0;
				foreach (FeatureInfo info in averageInfos.Values)
					overAllAverage += info.Average;
				overAllAverage /= averageInfos.Values.Count;
				return thisFeatureAverage / overAllAverage * 100;
			}
			else
				return double.NaN;
		}

		private Dictionary<Feature, FeatureInfo> CalcYearAveragePerFeature( ) {
			Dictionary<Feature, FeatureInfo> infos = new Dictionary<Feature, FeatureInfo>( );
			foreach (Vote vote in voteList) {
				FeatureInfo featureInfo = null;
				if (infos.ContainsKey(vote.Feature))
					featureInfo = infos[vote.Feature];
				else {
					featureInfo = new FeatureInfo( );
					infos[vote.Feature] = featureInfo;
				}
				featureInfo.AddValue(vote.Priority);
			}
			return infos;
		}

		class FeatureInfo {
			public void AddValue(int value) {
				prioritySum += value;
				valCount++;
			}
			private int prioritySum;
			public int PrioritySum {
				get {
					return prioritySum;
				}
			}
			private int valCount;
			public int ValCount {
				get {
					return valCount;
				}
			}
			public double Average {
				get {
					return ((double) prioritySum) / valCount;
				}
			}
		}

		public override bool CanResetValue(object component) { return false; }
		public override Type ComponentType { get { return typeof(Feature); } }
		public override bool IsReadOnly { get { return true; } }
		public override Type PropertyType { get { return typeof(double); } }
		public override void ResetValue(object component) { }
		public override bool ShouldSerializeValue(object component) { return false; }
	}
}
