﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Sturm.CollectionWrapping;

namespace FeatureVoting {
	public partial class FeatureVotingForm : Form {
		public FeatureVotingForm( ) {
			InitializeComponent( );

			CreateCustomGridStyles( );
		
			object dataSource = InitDemoData( );
			InitGrid(dataSource);
		}

		private void InitGrid(object dataSource) {
			FeatureView.AutoGenerateColumns = false;

			foreach (PropertyDescriptor propertyDescriptor in ((ITypedList) dataSource).GetItemProperties(null)) {
				DataGridViewColumn newColumn = null;

                if (propertyDescriptor is VoteValuePropertyDescriptor) {
					newColumn = new VoteValueColumn( );
				}
				else
					newColumn = new DataGridViewTextBoxColumn( );

				newColumn.DataPropertyName = propertyDescriptor.Name;
				newColumn.HeaderText = propertyDescriptor.DisplayName;
				FeatureView.Columns.Add(newColumn);
			}

			FeatureView.DataSource = dataSource;
		}

		private object InitDemoData( ) {
			BindingList<Feature> featureList = new BindingList<Feature>( );

			featureList.Add(new Feature("Synthesize out-of-the-box supply-chains"));
			featureList.Add(new Feature("Benchmark magnetic relationships"));
			featureList.Add(new Feature("Seize wireless web services"));
			featureList.Add(new Feature("Aggregate customized relationships"));
			featureList.Add(new Feature("Empower B2B platforms"));
			featureList.Add(new Feature("Incubate integrated bandwidth"));
			featureList.Add(new Feature("Syndicate vertical mindshare"));
			featureList.Add(new Feature("Implement world-class e-commerce"));
			featureList.Add(new Feature("Unleash impactful partnerships"));
			featureList.Add(new Feature("Engage ubiquitous paradigms"));
			featureList.Add(new SummaryFeature());

			BindingList<Vote> voteList = new BindingList<Vote>( );

			Random random = new Random( );
			for (int year = 2003; year < 2008; year++) {
				foreach (Feature feature in featureList)
					voteList.Add(new Vote(feature, year, random.Next(15)));
			}

			return TypedListWrapper.CreateWrapper(featureList, CalculatePropertyDescriptors(featureList, voteList));
		}

		IEnumerable<PropertyDescriptor> TypedEnumerate(PropertyDescriptorCollection propertyDescriptorCollection) {
			foreach (PropertyDescriptor propertyDescriptor in propertyDescriptorCollection) {
				yield return propertyDescriptor;
			}
		}

		private PropertyDescriptorCollection CalculatePropertyDescriptors(BindingList<Feature> featureList, BindingList<Vote> voteList) {
			PropertyDescriptorCollection origProperties = TypeDescriptor.GetProperties(typeof(Feature));

			List<PropertyDescriptor> properties = new List<PropertyDescriptor>(TypedEnumerate(origProperties));
				
			List<int> handledYears = new List<int>( );

			foreach (Vote vote in voteList)
				if (!handledYears.Contains(vote.Year)) {
					properties.Add(new VoteValuePropertyDescriptor(featureList, voteList, vote.Year));
					handledYears.Add(vote.Year);
				}

			properties.Add(new FeatureSummaryPropertyDescriptor(featureList, voteList));

			return new PropertyDescriptorCollection(properties.ToArray( ));
		}

		DataGridViewCellStyle rowHeaderStyle;
		DataGridViewCellStyle summaryColumnStyle;
		DataGridViewCellStyle summaryRowStyle;
		DataGridViewCellStyle crossHairStyle;

		private void CreateCustomGridStyles( ) {
			rowHeaderStyle = new DataGridViewCellStyle(FeatureView.ColumnHeadersDefaultCellStyle);
			rowHeaderStyle.SelectionBackColor = rowHeaderStyle.BackColor;
			rowHeaderStyle.SelectionForeColor = rowHeaderStyle.ForeColor;
			
			summaryColumnStyle = new DataGridViewCellStyle(FeatureView.DefaultCellStyle);
			summaryColumnStyle.BackColor = rowHeaderStyle.BackColor;
			summaryColumnStyle.SelectionBackColor = summaryColumnStyle.BackColor;
			summaryColumnStyle.SelectionForeColor = summaryColumnStyle.ForeColor;
			summaryColumnStyle.Font = new Font(summaryColumnStyle.Font, FontStyle.Bold);

			summaryRowStyle = new DataGridViewCellStyle(summaryColumnStyle);

			summaryColumnStyle.Format = "N2";

			crossHairStyle = new DataGridViewCellStyle(FeatureView.DefaultCellStyle);
			crossHairStyle.BackColor = SystemColors.Info;
		}

		private void FeatureView_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e) {
			if (FeatureView.Columns[e.ColumnIndex].DataPropertyName == "Name")
				e.CellStyle = rowHeaderStyle;
			else if (FeatureView.Columns[e.ColumnIndex].DataPropertyName == FeatureSummaryPropertyDescriptor.STR_PropertyName)
				e.CellStyle = summaryColumnStyle;
			else if (Attribute.IsDefined(((IList<Feature>) FeatureView.DataSource)[e.RowIndex].GetType( ), typeof(SummaryFeatureAttribute))) {
				int intVal = (int) e.Value;
				DataGridViewCellStyle cellStyle = null;
				if (intVal != 100) {
					cellStyle = new DataGridViewCellStyle(summaryRowStyle);
					cellStyle.ForeColor = intVal < 100 ? Color.Green : Color.Red;
				}
				else
					cellStyle = summaryRowStyle;
				e.CellStyle = cellStyle;
			}
			else if (e.ColumnIndex == FeatureView.CurrentCellAddress.X || e.RowIndex == FeatureView.CurrentCellAddress.Y)
				e.CellStyle = crossHairStyle;
		}

		int? oldRow = null;
		int? oldColumn = null;

		private void FeatureView_CellLeave(object sender, DataGridViewCellEventArgs e) {
			oldRow = e.RowIndex;
			oldColumn = e.ColumnIndex;
		}

		private void FeatureView_CellEnter(object sender, DataGridViewCellEventArgs e) {
			if (oldColumn.HasValue) {
				FeatureView.InvalidateColumn(oldColumn.Value);
				oldColumn = null;
			}
			if (oldRow.HasValue) {
				FeatureView.InvalidateRow(oldRow.Value);
				oldRow = null;
			}

			FeatureView.InvalidateRow(e.RowIndex);
			FeatureView.InvalidateColumn(e.ColumnIndex);
		}
	}
}