using System;
using System.ComponentModel;
using System.Collections.Generic;

namespace FeatureVoting {
	public class VoteValuePropertyDescriptor : PropertyDescriptor {
		public VoteValuePropertyDescriptor(BindingList<Feature> featureList, BindingList<Vote> voteList, int year)
			: base(year.ToString(), null) {
			this.featureList = featureList;
			this.voteList = voteList;
			voteList.ListChanged += new ListChangedEventHandler(voteList_ListChanged);
			this.year = year;
		}

		private BindingList<Feature> featureList;
		public BindingList<Feature> FeatureList {
			get {
				return featureList;
			}
		}

		private BindingList<Vote> voteList;
		public BindingList<Vote> VoteList {
			get {
				return voteList;
			}
		}

		void voteList_ListChanged(object sender, ListChangedEventArgs e) {
			if (e.ListChangedType == ListChangedType.ItemChanged) {
				PushChangeToSummaryFeatures( );
			}
		}

		private void PushChangeToSummaryFeatures( ) {
			foreach (Feature feature in featureList) {
				if (IsSummaryFeature(feature))
					feature.RaisePropertyChanged(year.ToString( ));
			}
		}

		private int year;
		public int Year {
			get {
				return year;
			}
		}

		public override void SetValue(object component, object value) {
			Feature feature = (Feature) component;
			if (!IsSummaryFeature(feature)) {
				Vote vote = GetVoteFor(voteList, feature, year);
				vote.Priority = (int) value;
			}
		}

		public override object GetValue(object component) {
			Feature feature = (Feature) component;
			if (IsSummaryFeature(feature)) {
				return CalcPrioritySumForYear( );
			}
			else {
				Vote vote = GetVoteFor(voteList, feature, year);
				return vote.Priority;
			}
		}

		private int CalcPrioritySumForYear(BindingList<Vote> voteList, int year) {
			int sum = 0;

			foreach (Vote vote in voteList) {
				if (vote.Year == year)
					sum += vote.Priority;
			}

			return sum;
		}

		public int CalcPrioritySumForYear( ) {
			return CalcPrioritySumForYear(this.voteList, this.year);
		}

		private bool IsSummaryFeature(Feature feature) {
			Type featureType = feature.GetType( );
			return Attribute.IsDefined(featureType, typeof(SummaryFeatureAttribute));
		}

		private Vote GetVoteFor(IList<Vote> voteList, Feature feature, int year) {
			foreach (Vote vote in voteList) {
				if (vote.Feature == feature && vote.Year == year)
					return vote;
			}
			Vote newVote = new Vote(feature, year, 0);
			voteList.Add(newVote);
			return newVote;
		}

		public override bool CanResetValue(object component) { return false; }
		public override Type ComponentType { get { return typeof(Feature); } }
		public override bool IsReadOnly { get { return false; } }
		public override Type PropertyType { get { return typeof(int); } }
		public override void ResetValue(object component) { }
		public override bool ShouldSerializeValue(object component) { return false; }
	}


}
